package it.zanichelli.android.esercizio21;

/**
 * Created by Giorgio Meini on 17/03/2017.
 */

import java.io.*;
import java.net.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

public class EuroConverter {
  final private String url = "http://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml";
  String currency;
  private boolean success;
  private float rate;

  public EuroConverter(String currency) {
    this.currency = currency;
    this.success = false;
    this.rate = 0.0f;
  }

  public boolean askRate() {
    try {
      // invocke WS
      URL server = new URL(url);
      HttpURLConnection service = (HttpURLConnection)server.openConnection();
      //service.setRequestProperty("Host", "http://www.ecb.europa.eu");
      service.setRequestProperty("Accept", "application/xml");
      service.setRequestProperty("Accept-Charset", "UTF-8");
      service.setDoInput(true);
      service.setRequestMethod("GET");
      service.setReadTimeout(1000); // timeout: 1s
      service.connect();
      int status = service.getResponseCode();
      if (status != 200) {
        return false;
      }
      // parse XML
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document document = builder.parse(service.getInputStream());
      Element root = document.getDocumentElement();
      NodeList first = root.getElementsByTagName("Cube");
      if (first != null && first.getLength() > 0) {
        Element cube = (Element)first.item(0);
        NodeList second = cube.getElementsByTagName("Cube");
        if (second != null && second.getLength() > 0) {
          cube = (Element)second.item(0);
          NodeList cubes = cube.getElementsByTagName("Cube");
          if (cubes != null && cubes.getLength() > 0) {
            for (int n=0; n<cubes.getLength(); n++) {
              NamedNodeMap attributes = cubes.item(n).getAttributes();
              Node _currency = attributes.getNamedItem("currency");
              if (currency.equalsIgnoreCase(_currency.getTextContent())) {
                Node _rate = attributes.getNamedItem("rate");
                rate = Float.valueOf(_rate.getTextContent());
                service.getInputStream().close();
                success = true;
                return true;
              }
            }
          }
        }
      }
      service.getInputStream().close();
      return false;
    }
    catch (IOException |  ParserConfigurationException | SAXException exception) {
      return false;
    }
  }

  public float getRate() {
    if (!success)
      return 0.0f;
    else
      return rate;
  }

  /*
  // Esempio di utilizzo della classe
  public static void main(String args[]) {

    float rate;

    EuroConverter euroconverter = new EuroConverter("USD");
    euroconverter.askRate();
    if ((rate = euroconverter.getRate()) == 0.0f)
      System.err.println("Errore!");
    else
      System.out.println("Rate="+rate);
  }

  */
}