package it.zanichelli.android.esercizio21;

import android.content.SharedPreferences;
import android.preference.PreferenceManager;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;

import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.TextView;
import android.widget.Toast;

public class MainActivity extends AppCompatActivity implements OnEuroConveterListener, RadioGroup.OnCheckedChangeListener {

  private AlertDialog settingsDialog;

  private SharedPreferences sharedPreferences;

  // stato corrente del convertitore
  private float currentRate;
  private int currentCurrencyId;

  // diverse visualizzazioni possibili
  private LinearLayout normalLayout;
  private TextView connectingMessage;
  private LinearLayout errorLayout;

  // elementi di normalLayout il cui contenuto cambia nel tempo
  private EditText value;
  private Button toEur;
  private Button fromEur;
  private TextView resultText;

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_main);

    sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

    currentCurrencyId = sharedPreferences.getInt(getString(R.string.pref_currency_key), R.string.usd);

    normalLayout      = (LinearLayout)  findViewById(R.id.normal_layout);
    connectingMessage = (TextView)      findViewById(R.id.connecting_message);
    errorLayout       = (LinearLayout)  findViewById(R.id.error_layout);

    value       = (EditText)  findViewById(R.id.value);
    toEur       = (Button)    findViewById(R.id.to_eur);
    fromEur     = (Button)    findViewById(R.id.from_eur);
    resultText  = (TextView)  findViewById(R.id.result);

    currentRate = 0.f;

    AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);

    settingsDialog = dialogBuilder
      .setTitle(R.string.pref_title)
      .setView(R.layout.alert_preference)
      .setCancelable(true)
      .setNegativeButton(R.string.cancel, null)
      .create();
  }


  // apre la finestra di impostazioni
  void openPreferences() {
    settingsDialog.show();

    RadioGroup radiogroup = (RadioGroup) settingsDialog.findViewById(R.id.pref_radio_group);

    RadioButton currentButton =
      (RadioButton) radiogroup.findViewById(stringIdToButtonId(currentCurrencyId));
    currentButton.setChecked(true);

    radiogroup.setOnCheckedChangeListener(this);
  }

  // imposta la visualizzazione corrente
  private void setInterface(int id) {
    normalLayout.setVisibility(View.GONE);
    connectingMessage.setVisibility(View.GONE);
    errorLayout.setVisibility(View.GONE);

    switch (id) {
      case R.id.normal_layout:
        normalLayout.setVisibility(View.VISIBLE);
        break;
      case R.id.connecting_message:
        connectingMessage.setVisibility(View.VISIBLE);
        break;
      case R.id.error_layout:
        errorLayout.setVisibility(View.VISIBLE);
        break;
    }
  }

  // aggiorna il contenuto degli elementi di normalLayout
  void updateLayout() {
    value.setText("");
    toEur.setText(getString(currentCurrencyId) + " " + getString(R.string.to_base) + " " + getString(R.string.eur));
    fromEur.setText(getString(R.string.eur) + " " + getString(R.string.to_base) + " " + getString(currentCurrencyId));
    resultText.setText("");
  }

  // richiede il rate di conversione per la valuta corrente
  private void requestRate() {
    String curr = getString(currentCurrencyId);
    EuroConverterAsync.requestEuroConverter(curr, this);
  }

  // quando la app viene attivata, richiede il rate di
  // conversione per la valuta corrente
  @Override
  protected void onStart() {
    super.onStart();

    requestRate();
  }

  // mostra un messaggio di attesa a video
  @Override
  public void onEuroConverterStart() {
    setInterface(R.id.connecting_message);
  }

  // se il rate ottenuto è diverso da zero, allora imposta la visualizzazione
  // del normalLayout con il giusto contenuto dei widget
  @Override
  public void onEuroConverterResult(float rate) {
    if (rate == 0.f) {
      setInterface(R.id.error_layout);
      return;
    }

    currentRate = rate;
    setInterface(R.id.normal_layout);
    updateLayout();
  }

  // esegue una conversione da/per euro per/da la valuta correntemente selezionata
  // l'argomento è true se la conversione è dalla valuta selezionata a euro, false altrimenti
  private void updateResult(boolean toEur) {
    double resultValue;
    double requestedValue;

    try {
      requestedValue = Double.parseDouble(value.getText().toString());
    } catch (NumberFormatException ex) {
      // il campo di testo è vuoto o il valore inserito non è un numero corretto
      Toast.makeText(this, R.string.error, Toast.LENGTH_LONG).show();
      return;
    }

    StringBuilder stringBuilder = new StringBuilder();

    if (toEur) {
      resultValue = requestedValue / currentRate;

      stringBuilder
        .append(requestedValue)
        .append(" ")
        .append(getString(currentCurrencyId))
        .append(" ")
        .append(getString(R.string.message_link))
        .append(" ")
        .append(resultValue)
        .append(" ")
        .append(getString(R.string.eur));
    } else {
      resultValue = requestedValue * currentRate;

      stringBuilder
        .append(requestedValue)
        .append(" ")
        .append(getString(R.string.eur))
        .append(" ")
        .append(getString(R.string.message_link))
        .append(" ")
        .append(resultValue)
        .append(" ")
        .append(getString(currentCurrencyId));
    }

    resultText.setText(stringBuilder.toString());
  }

  protected void onClick(View v) {
    switch (v.getId()) {
      case R.id.to_eur:
        updateResult(true);
        break;
      case R.id.from_eur:
        updateResult(false);
        break;
      case R.id.settings:
        openPreferences();
        break;
      case R.id.retry:
        // riprova a ottenere il rate per la valuta corrente
        // solo in caso di un fallimento precedente nell'interrogazione
        // del web-service
        requestRate();
        break;
    }
  }

  // converte l'intero che identifica il radio-button con l'intero che
  // identifica la risorsa stringa corrispondente
  private int buttonIdToStringId(int buttonId) {
    int stringId = -1;

    switch (buttonId) {
      case R.id.usd:
        stringId = R.string.usd;
        break;
      case R.id.gbp:
        stringId = R.string.gbp;
        break;
      case R.id.jpy:
        stringId = R.string.jpy;
        break;
      case R.id.chf:
        stringId = R.string.chf;
        break;
      case R.id.rub:
        stringId = R.string.rub;
        break;
      case R.id.cny:
        stringId = R.string.cny;
        break;
    }

    return stringId;
  }

  // esegue l'operazione inversa di buttonIdToStringId
  private int stringIdToButtonId(int stringId) {
    int buttonId = -1;

    switch (stringId) {
      case R.string.usd:
        buttonId = R.id.usd;
        break;
      case R.string.gbp:
        buttonId = R.id.gbp;
        break;
      case R.string.jpy:
        buttonId = R.id.jpy;
        break;
      case R.string.chf:
        buttonId = R.id.chf;
        break;
      case R.string.rub:
        buttonId = R.id.rub;
        break;
      case R.string.cny:
        buttonId = R.id.cny;
        break;
    }

    return buttonId;
  }

  // al cambiamento di un valore nella finestra delle preferenze aggiorna
  // le preferenze e richiede il nuovo rate di conversione
  @Override
  public void onCheckedChanged(RadioGroup radioGroup, int radioButtonId) {
    currentCurrencyId = buttonIdToStringId(radioButtonId);

    SharedPreferences.Editor editor =
      sharedPreferences.edit();
    editor.putInt(
      getString(R.string.pref_currency_key), currentCurrencyId);
    editor.apply();

    settingsDialog.cancel();

    requestRate();
  }
}

