package com.zanichelli.android.reversegeocoder;

import android.content.Context;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.support.v7.app.AlertDialog;
import android.os.Bundle;
import android.view.View;
import android.widget.TextView;

public class MainActivity extends PermissionCompatActivity implements OnReverseGeocodingListener {
  private static final int REQUEST_LOCATION_LAST = 0;
  private boolean hasPendingRequest = false;

  private TextView value;
  private AlertDialog error;

  private LocationListener listener = new LocationListener() {
    @Override
    public void onLocationChanged(Location location) {
      // aggiorna il valore visualizzato alla ricezione di una nuova posizione
      updateLocation(location);
    }

    @Override
    public void onStatusChanged(String s, int i, Bundle bundle) {
    }

    @Override
    public void onProviderEnabled(String s) {
    }

    @Override
    public void onProviderDisabled(String s) {
    }
  };

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_main);

    value = (TextView) findViewById(R.id.value);

    error = new AlertDialog.Builder(this)
      .setTitle(R.string.alert_title)
      .setMessage(R.string.alert_message)
      .setPositiveButton(R.string.alert_positive, null)
      .setCancelable(true)
      .create();
  }

  @Override
  protected void onStart() {
    super.onStart();

    // richiede un aggiornamento della posizione quando la activity diventa attiva
    requestLocationUpdate();
  }

  @Override
  protected void onStop() {
    super.onStop();

    // se un aggiornamento era stato precedentemente richiesto, questo viene annullato
    if (hasPendingRequest) {
      LocationManager lm = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
      try {
        lm.removeUpdates(listener);
      } catch (SecurityException ex) {

      }
      hasPendingRequest = false;
    }
  }

  // metodo invocato alla ricezione dell'aggiornamento richiesto
  void updateLocation(Location location) {
    value.setText("Latitudine: "
      + location.getLatitude()
      + "\n" +
      "Longitudine: "
      + location.getLongitude());

    // aichiesta asincrona di traduzione dell'indirizzo
    ReverseGeocoder.requestAddressFromLocation(
      REQUEST_LOCATION_LAST, location, this);

    hasPendingRequest = false;
  }

  // metodo invocato alla pressione del pulsante
  void requestLocationUpdate(View v) {
    requestLocationUpdate();
  }

  void requestLocationUpdate() {
    if(hasPendingRequest)
      return;

    // se la activity non possiede il permesso necessario
    // l'aggiornamento viene posticipato
    if (!checkLocationPermission())
      return;

    LocationManager lm = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
    try {
      lm.requestSingleUpdate(LocationManager.GPS_PROVIDER, listener, null);
      hasPendingRequest = true;
    } catch (SecurityException ex) {

    }
  }

  // alla ricezione del risultato dell'invocazione del web-service la posizione
  // visualizzata viene aggiornata
  @Override
  public void onReverseGeocodingResult(int requestCode, String result) {
    if (result == null)
      return;

    value.setText(result);
  }

  // se l'utente concede il permesso ad accedere al GPS viene richeista la ricezione
  // dell'aggiornamento annullata in precedenza
  @Override
  public void onPermissionGranted(int requestCode) {
    super.onPermissionGranted(requestCode);
    requestLocationUpdate();
  }

  // se l'utente non concede il permesso,  mostra una finestra che motiva
  // la richiesta effettuata
  @Override
  public void onPermissionDenied(int requestCode) {
    error.show();
  }
}
