package com.zanichelli.android.reversegeocoder;

import android.Manifest;
import android.content.pm.PackageManager;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;

public class PermissionCompatActivity extends AppCompatActivity {
  protected static final int PERMISSION_REQUEST_ACCESS_FINE_LOCATION = 1;
  protected static final int PERMISSION_REQUEST_ACCESS_COARSE_LOCATION = 2;

  public boolean checkLocationPermission() {
    return checkPermission(Manifest.permission.ACCESS_FINE_LOCATION) &&
      checkPermission(Manifest.permission.ACCESS_COARSE_LOCATION);
  }

  // controlla se la APP possiede il permesso richiesto in caso
  // contrario effettua anche la richiesta di tale permesso
  public boolean checkPermission(String permission) {
    if (ContextCompat.checkSelfPermission(
      this, permission)
      == PackageManager.PERMISSION_GRANTED)
      return true;

    int requestCode;
    switch (permission) {
      case Manifest.permission.ACCESS_FINE_LOCATION:
        requestCode = PERMISSION_REQUEST_ACCESS_FINE_LOCATION;
        break;
      case Manifest.permission.ACCESS_COARSE_LOCATION:
        requestCode = PERMISSION_REQUEST_ACCESS_COARSE_LOCATION;
        break;
      default:
        throw new RuntimeException(
          "You cannot request a permission not shown in the manifest file.\n"
            + "You requested " + permission);
    }

    ActivityCompat.requestPermissions(this,
      new String[]{permission},
      requestCode);

    return false;
  }

  // invocato dal sistema dopo aver chiesto all'utente
  // se rilasciare o meno il permesso alla APP
  @Override
  public void onRequestPermissionsResult(
    int requestCode, String permissions[], int[] grantResults) {

    switch (requestCode) {
      case PERMISSION_REQUEST_ACCESS_FINE_LOCATION:
      case PERMISSION_REQUEST_ACCESS_COARSE_LOCATION:
        if (grantResults.length > 0
          && grantResults[0] == PackageManager.PERMISSION_GRANTED) {

          this.onPermissionGranted(requestCode);

        } else {

          this.onPermissionDenied(requestCode);
        }
        return;
    }
  }

  // invocato quando una richiesta di un permesso
  // è terminata con esito negativo
  public void onPermissionDenied(int requestCode) {
  }

  // invocato quando una richiesta di un permesso
  // è terminata con esito positivo
  public void onPermissionGranted(int requestCode) {
  }
}
