package com.zanichelli.android.reversegeocoder;

import android.location.Location;
import android.os.AsyncTask;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

public class ReverseGeocoder {

  // metodo statico per avviare una richiesta asincrona
  static void requestAddressFromLocation(
    int requestCode,
    Location location,
    OnReverseGeocodingListener listener) {

    ReverseGeocoderTask task = new ReverseGeocoderTask(requestCode, listener);
    task.execute(location);
  }
}

class ReverseGeocoderTask extends AsyncTask<Location, Void, String> {
  private int requestCode;
  private OnReverseGeocodingListener listener;

  private final int TIMEOUT = 1000; // 1s
  private final String prefix =
    "http://maps.googleapis.com/maps/api/geocode/xml?latlng=";
  private final String host = "maps.googleapis.com";

  // eccezione sollevata in caso di errore del reverse geocoding
  private class ReverseGeocodingException extends Exception {
  }

  public ReverseGeocoderTask(int request, OnReverseGeocodingListener lis) {
    super();
    requestCode = request;
    listener = lis;
  }

  // parser della risposta XML ricevuta dal web-service
  private String parseXML(InputStream in)
    throws ReverseGeocodingException {
    String address = null;

    try {
      DocumentBuilderFactory factory =
        DocumentBuilderFactory.newInstance();
      DocumentBuilder documentBuilder =
        factory.newDocumentBuilder();

      Document document = documentBuilder.parse(in);

      Element root = document.getDocumentElement();
      NodeList list = root.getElementsByTagName("status");
      if (list != null && list.getLength() > 0) {
        if (list.item(0).getFirstChild()
          .getNodeValue().equalsIgnoreCase("OK")) {

          list = root.getElementsByTagName("formatted_address");
          if (list != null && list.getLength() > 0) {
            address = list.item(0).getFirstChild().getNodeValue();
          }
        }
      }
    } catch (IOException e) {
      throw new ReverseGeocodingException();
    } catch (ParserConfigurationException e) {
      throw new ReverseGeocodingException();
    } catch (SAXException e) {
      throw new ReverseGeocodingException();
    }

    return address;
  }

  // l'indirizzo richiesto è ottenuto tramite interrogazione
  // dello specifico web-service di Google per il reverse geocoding
  private String getAddress(Location location)
    throws ReverseGeocodingException {

    try {
      String url = prefix +
        URLEncoder.encode(Double.toString(location.getLatitude()), "UTF-8")
        + "," +
        URLEncoder.encode(Double.toString(location.getLongitude()), "UTF-8");

      URL server = new URL(url);

      HttpURLConnection service =
        (HttpURLConnection) server.openConnection();
      service.setRequestMethod("GET");
      service.setRequestProperty("Host", host);
      service.setRequestProperty("Accept", "application/xml");
      service.setRequestProperty("Accept-Charset", "UTF-8");
      service.setConnectTimeout(TIMEOUT);
      service.setDoInput(true);
      service.setReadTimeout(TIMEOUT);
      service.connect();

      int status = service.getResponseCode();

      if (status != 200) {
        throw new ReverseGeocodingException();
      }

      return parseXML(service.getInputStream());

    } catch (IOException e) {
      throw new ReverseGeocodingException();
    }
  }

  @Override
  protected String doInBackground(Location... params) {
    String address = null;
    try {
      address = getAddress(params[0]);
    } catch (ReverseGeocodingException ex) {

    }

    // nel caso di fallimento della richiesta, il valore restituito è null
    return address;
  }

  @Override
  protected void onPostExecute(String result) {
    // al termine dell'esecuzione, il valore restituito
    // dal metodo doInBackground viene inviato al listener registrato
    listener.onReverseGeocodingResult(requestCode, result);
  }
}